import {
  UserCtor,
  UserInfo,
  UserInfoWithPassword,
  UserRoles,
  UserWithPasswordCtor,
} from '@core';
import { getHashedPassword } from '../functions/password';
import { Entity } from './entity';

export class User extends Entity implements UserInfo {
  username: string;
  role: keyof typeof UserRoles;

  constructor(raw: UserCtor) {
    super(raw);

    this.username = raw.username || '';
    this.role = raw.role || UserRoles.USER;
  }

  Info(): UserInfo {
    return {
      uuid: this.uuid,
      username: this.username,
      role: this.role,
    };
  }
}

export class UserWithPassword extends User implements UserInfoWithPassword {
  password: string;

  constructor(raw: UserWithPasswordCtor) {
    super(raw);

    this.password = getHashedPassword(this.uuid, raw.password || '');
  }

  Info(): UserInfoWithPassword {
    return {
      uuid: this.uuid,
      username: this.username,
      role: this.role,
      password: this.password,
    };
  }
}
